/*
 VARNA is a tool for the automated drawing, visualization and annotation of the secondary structure of RNA, designed as a companion software for web servers and databases.
 Copyright (C) 2008  Kevin Darty, Alain Denise and Yann Ponty.
 electronic mail : Yann.Ponty@lri.fr
 paper mail : LRI, bat 490 Universit Paris-Sud 91405 Orsay Cedex France

 This file is part of VARNA version 3.1.
 VARNA version 3.1 is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

 VARNA version 3.1 is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 See the GNU General Public License for more details.

 You should have received a copy of the GNU General Public License along with VARNA version 3.1.
 If not, see http://www.gnu.org/licenses.
 */
import java.awt.GridLayout;
import java.io.IOException;
import java.util.ArrayList;

import javax.swing.JApplet;
import javax.swing.JOptionPane;

import fr.orsay.lri.varna.VARNAPanel;
import fr.orsay.lri.varna.exceptions.ExceptionFileFormatOrSyntax;
import fr.orsay.lri.varna.exceptions.ExceptionModeleStyleBaseSyntaxError;
import fr.orsay.lri.varna.exceptions.ExceptionNonEqualLength;
import fr.orsay.lri.varna.exceptions.ExceptionParameterError;
import fr.orsay.lri.varna.interfaces.InterfaceParameterLoader;
import fr.orsay.lri.varna.models.VARNAConfigLoader;



public class VARNA extends JApplet implements InterfaceParameterLoader {
	ArrayList<VARNAPanel> _vpl = null;
	/**
	 * 
	 */
	private static final long serialVersionUID = -2598221520127067670L;

	public VARNA() {
		super();
	}

	public void init() {
		try {
			VARNAConfigLoader VARNAcfg = new VARNAConfigLoader(this);
			
			try {
				_vpl = VARNAcfg.createVARNAPanels();
			} catch (IOException e) {
				JOptionPane.showMessageDialog(this, e.getMessage(),
						"VARNA Error", JOptionPane.ERROR_MESSAGE);
			} catch (ExceptionFileFormatOrSyntax e) {
				JOptionPane.showMessageDialog(this, e.getMessage(),
						"VARNA Error", JOptionPane.ERROR_MESSAGE);
			}
			setLayout(new GridLayout(VARNAcfg.getNbColumns(), VARNAcfg
					.getNbRows()));
			for (int i = 0; i < _vpl.size(); i++) {
				getContentPane().add(_vpl.get(i));
			}
			getContentPane().setVisible(true);
			getContentPane().repaint();
		} catch (ExceptionParameterError e) {
			VARNAPanel.errorDialogStatic(e, this);
		} catch (ExceptionModeleStyleBaseSyntaxError e) {
			VARNAPanel.errorDialogStatic(e, this);
		} catch (ExceptionNonEqualLength e) {
			VARNAPanel.errorDialogStatic(e, this);
		}

	}

	public String getParameterValue(String key, String def) {
		if (getParameter(key) == null) {
			return def;
		} else {
			return getParameter(key);
		}
	}

	public String[][] getParameterInfo() {
		return VARNAConfigLoader.getParameterInfo();
	}
	
	public ArrayList<VARNAPanel> getPanels()
	{
		return _vpl;
	}

}
