/*
 VARNA is a tool for the automated drawing, visualization and annotation of the secondary structure of RNA, designed as a companion software for web servers and databases.
 Copyright (C) 2008  Kevin Darty, Alain Denise and Yann Ponty.
 electronic mail : Yann.Ponty@lri.fr
 paper mail : LRI, bat 490 Universit Paris-Sud 91405 Orsay Cedex France

 This file is part of VARNA version 3.1.
 VARNA version 3.1 is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

 VARNA version 3.1 is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 See the GNU General Public License for more details.

 You should have received a copy of the GNU General Public License along with VARNA version 3.1.
 If not, see http://www.gnu.org/licenses.
 */
package fr.orsay.lri.varna.views;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.Point;
import java.awt.event.KeyEvent;
import java.util.ArrayList;

import javax.swing.JCheckBoxMenuItem;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JSeparator;
import javax.swing.KeyStroke;

import fr.orsay.lri.varna.VARNAPanel;
import fr.orsay.lri.varna.controlers.ControleurMenu;
import fr.orsay.lri.varna.models.rna.RNA;

public class VueMenu extends JPopupMenu {
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	private VARNAPanel _vp;

	private ControleurMenu _controleurMenu;

	private JCheckBoxMenuItem _itemOptionSpecialBaseColored = new JCheckBoxMenuItem(
			"Custom colored", false);
	private JCheckBoxMenuItem _itemShowWarnings = new JCheckBoxMenuItem(
			"Show warnings", false);
	private JCheckBoxMenuItem _itemOptionGapsBaseColored = new JCheckBoxMenuItem(
			"Custom colored", false);
	private JCheckBoxMenuItem _itemOptionBondsColored = new JCheckBoxMenuItem(
			"Use base color for base-pairs", false);
	private JCheckBoxMenuItem _itemShowNCBP = new JCheckBoxMenuItem(
			"Show non-canonical BPs", true);
	private JCheckBoxMenuItem _itemShowOnlyPlanar = new JCheckBoxMenuItem(
			"Hide tertiary BPs", false);
	private JCheckBoxMenuItem _itemFlatExteriorLoop = new JCheckBoxMenuItem(
			"Flat exterior loop", false);
	private JMenuItem _dashBasesColor;

	private ArrayList<JComponent> _disabled = new ArrayList<JComponent>();

	private JMenuItem _rotation;
	private JMenuItem _bpHeightIncrement;

	private Point _spawnOrigin = new Point(-1,-1);
	
	public VueMenu(VARNAPanel vp) {
		_vp = vp;
		_controleurMenu = new ControleurMenu(_vp, this);
	}

	private void addTitle(String title, boolean keep) {
		JSeparator sep = new JSeparator();
		JLabel titleItem = new JLabel(" " + title);
		// titleItem.setAlignmentX(0.5f);
		Font previousFont = titleItem.getFont();
		Font futureFont = previousFont.deriveFont(Font.BOLD).deriveFont(
				(float) previousFont.getSize() + 1.0f);

		titleItem.setFont(futureFont);
		Color current = titleItem.getForeground();
		Color future = current.brighter().brighter();
		// titleItem.setBackground(future);
		titleItem.setForeground(future);
		add(titleItem);
		add(sep);
		if (!keep) {
			_disabled.add(sep);
			_disabled.add(titleItem);
		}
	}

	/**
	 * Build the popup menu
	 */
	public void buildPopupMenu() {
		addTitle("File", true);
		fileMenu();
		exportMenu();
		addSeparator();

		addTitle("Display", true);
		viewMenu();
		displayMenu();
		JSeparator sep = new JSeparator();
		add(sep);
		_disabled.add(sep);

		addTitle("Edit", false);
		redrawMenu();
		colorClassesMenu();
		annotationMenu();
		_disabled.add(_itemShowNCBP);
		_disabled.add(_itemShowOnlyPlanar);
		aboutMenu();
	}

	private void annotationMenu() {
		JMenu submenuAnnotations = new JMenu("Annotations");

		JMenuItem annotationAdd = new JMenuItem("Add here");
		annotationAdd.setActionCommand("annotationsadd");
		annotationAdd.addActionListener(_controleurMenu);
		submenuAnnotations.add(annotationAdd);

		JMenuItem annotationEdit = new JMenuItem("Edit...");
		annotationEdit.setActionCommand("annotationsedit");
		annotationEdit.addActionListener(_controleurMenu);
		submenuAnnotations.add(annotationEdit);

		JMenuItem annotationRemove = new JMenuItem("Remove...");
		annotationRemove.setActionCommand("annotationsremove");
		annotationRemove.addActionListener(_controleurMenu);
		submenuAnnotations.add(annotationRemove);
		
		submenuAnnotations.addSeparator();
		
		JMenuItem itemAutoHelices = new JMenuItem("Auto helices");
		itemAutoHelices.setActionCommand("annotationsautohelices");
		itemAutoHelices.setAccelerator(KeyStroke.getKeyStroke("control Q"));
		itemAutoHelices.addActionListener(_controleurMenu);
		submenuAnnotations.add(itemAutoHelices);

		JMenuItem itemAutoInterior = new JMenuItem("Auto interior loops");
		itemAutoInterior.setActionCommand("annotationsautointerior");
		itemAutoInterior.setAccelerator(KeyStroke.getKeyStroke("alt shift Q"));
		itemAutoInterior.addActionListener(_controleurMenu);
		submenuAnnotations.add(itemAutoInterior);

		JMenuItem itemAutoTerminal = new JMenuItem("Auto terminal loops");
		itemAutoTerminal.setActionCommand("annotationsautoterminal");
		itemAutoTerminal.setAccelerator(KeyStroke.getKeyStroke("control shift Q"));
		itemAutoTerminal.addActionListener(_controleurMenu);
		submenuAnnotations.add(itemAutoTerminal);

		add(submenuAnnotations);
	}

	private void aboutMenu() {
		JMenuItem submenuAbout = new JMenuItem("About VARNA");
		submenuAbout.setActionCommand("about");
		submenuAbout.setAccelerator(KeyStroke.getKeyStroke("control A"));
		submenuAbout.addActionListener(_controleurMenu);
		addSeparator();
		add(submenuAbout);
	}

	private void fileMenu() {
		// JMenu submenuFile = new JMenu("File");
		JMenuItem submenuNew = new JMenuItem("New...");
		submenuNew.setActionCommand("userInput");
		submenuNew.setAccelerator(KeyStroke.getKeyStroke("control N"));
		submenuNew.addActionListener(_controleurMenu);
		_disabled.add(submenuNew);

		JMenuItem submenuOpen = new JMenuItem("Open...");
		submenuOpen.setAccelerator(KeyStroke.getKeyStroke("control O"));
		submenuOpen.setActionCommand("file");
		submenuOpen.addActionListener(_controleurMenu);
		_disabled.add(submenuOpen);

		// Save secondary structure
		JMenu submenuSave = new JMenu("Save as");
		submenuSave.setMnemonic(KeyEvent.VK_S);
		JMenuItem exportDBN = new JMenuItem("DBN (Vienna)");
		exportDBN.setActionCommand("dbn");
		exportDBN.addActionListener(_controleurMenu);
		submenuSave.add(exportDBN);

		JMenuItem exportBPSEQ = new JMenuItem("BPSEQ");
		exportBPSEQ.setActionCommand("bpseq");
		exportBPSEQ.addActionListener(_controleurMenu);
		submenuSave.add(exportBPSEQ);

		JMenuItem exportCT = new JMenuItem("CT");
		exportCT.setActionCommand("ct");
		exportCT.addActionListener(_controleurMenu);
		submenuSave.add(exportCT);

		add(submenuNew);
		add(submenuOpen);
		add(submenuSave);
	}

	private void exportMenu() {
		// Export menu
		JMenu submenuExport = new JMenu("Export");
		submenuExport.setMnemonic(KeyEvent.VK_X);

		JMenuItem exportEPS = new JMenuItem("EPS");
		exportEPS.setActionCommand("eps");
		exportEPS.addActionListener(_controleurMenu);

		JMenuItem exportXFIG = new JMenuItem("XFIG");
		exportXFIG.setActionCommand("xfig");
		exportXFIG.addActionListener(_controleurMenu);

		JMenuItem exportSVG = new JMenuItem("SVG");
		exportSVG.setActionCommand("svg");
		exportSVG.addActionListener(_controleurMenu);

		JMenuItem exportPNG = new JMenuItem("PNG");
		exportPNG.setActionCommand("png");
		exportPNG.addActionListener(_controleurMenu);

		JMenuItem exportJPEG = new JMenuItem("JPEG");
		exportJPEG.setActionCommand("jpeg");
		exportJPEG.addActionListener(_controleurMenu);

		submenuExport.add(exportSVG);
		submenuExport.add(exportXFIG);
		submenuExport.addSeparator();
		submenuExport.add(exportEPS);
		submenuExport.addSeparator();
		submenuExport.add(exportPNG);
		submenuExport.add(exportJPEG);
		add(submenuExport);
	}

	public void updateDialog() {
		for (int i = 0; i < _disabled.size(); i++) {
			JComponent j = _disabled.get(i);
			j.setVisible(_vp.isModifiable());
		}
		_itemOptionSpecialBaseColored.setState(_vp.getColorSpecialBases());
		_itemShowWarnings.setState(_vp.getShowWarnings());
		_itemOptionGapsBaseColored.setState(_vp.getColorGapsBases());
		_itemOptionGapsBaseColored.setEnabled(_vp.isComparisonMode());
		_dashBasesColor.setEnabled(_vp.isComparisonMode());
		_bpHeightIncrement
				.setEnabled(_vp.getDrawMode() == RNA.DRAW_MODE_LINEAR);
		_itemOptionBondsColored.setState(_vp.getUseBaseColorsForBPs());
		_itemShowNCBP.setState(_vp.getShowNonCanonicalBP());
		_itemShowOnlyPlanar.setState(!_vp.getShowNonPlanarBP());
		_itemFlatExteriorLoop.setState(_vp.getFlatExteriorLoop());
		_itemFlatExteriorLoop
				.setEnabled(_vp.getDrawMode() == RNA.DRAW_MODE_RADIATE);
	}

	private void displayMenu() {
		// Export menu
		JMenu submenuStyle = new JMenu("RNA style");

		JMenu subMenuMisc = new JMenu("Misc");

		// Menu Title
		JMenu subMenuTitle = new JMenu("Title");

		JMenuItem title = new JMenuItem("Set title...");
		title.setActionCommand("setTitle");
		title.setAccelerator(KeyStroke.getKeyStroke("control T"));
		title.addActionListener(_controleurMenu);
		subMenuTitle.add(title);
		_disabled.add(title);

		JMenuItem titleDisplay = new JMenuItem("Font");
		titleDisplay.setActionCommand("titleDisplay");
		titleDisplay.setAccelerator(KeyStroke.getKeyStroke("control shift T"));
		titleDisplay.addActionListener(_controleurMenu);
		subMenuTitle.add(titleDisplay);
		_disabled.add(titleDisplay);

		JMenuItem titleColor = new JMenuItem("Color");
		titleColor.setActionCommand("titleColor");
		titleColor.setAccelerator(KeyStroke.getKeyStroke("control alt T"));
		titleColor.addActionListener(_controleurMenu);
		subMenuTitle.add(titleColor);
		_disabled.add(titleColor);
		_disabled.add(subMenuTitle);

		JMenuItem submenuBPStyle = new JMenuItem("Base-pairs style...");
		submenuBPStyle.setAccelerator(KeyStroke.getKeyStroke("control P"));
		submenuBPStyle.setMnemonic(KeyEvent.VK_P);
		submenuBPStyle.setActionCommand("bpstyle");
		submenuBPStyle.addActionListener(_controleurMenu);

		JMenu subMenuNSBases = new JMenu("Non-standard bases");

		_itemOptionSpecialBaseColored.setActionCommand("specialbasecolored");
		_itemOptionSpecialBaseColored.setAccelerator(KeyStroke
				.getKeyStroke("control J"));
		_itemOptionSpecialBaseColored.addActionListener(_controleurMenu);

		JMenuItem specialBasesColor = new JMenuItem("Color");
		specialBasesColor.setActionCommand("specialBasesColor");
		specialBasesColor.setAccelerator(KeyStroke
				.getKeyStroke("control shift J"));
		specialBasesColor.addActionListener(_controleurMenu);

		subMenuNSBases.add(_itemOptionSpecialBaseColored);
		subMenuNSBases.add(specialBasesColor);

		JMenu subMenuGapsBases = new JMenu("'Gaps' bases");

		_itemOptionGapsBaseColored.setActionCommand("dashbasecolored");
		_itemOptionGapsBaseColored.setAccelerator(KeyStroke
				.getKeyStroke("control D"));
		_itemOptionGapsBaseColored.addActionListener(_controleurMenu);
		_dashBasesColor = new JMenuItem("Color");
		_dashBasesColor.setActionCommand("dashBasesColor");
		_dashBasesColor.setAccelerator(KeyStroke
				.getKeyStroke("control shift D"));
		_dashBasesColor.addActionListener(_controleurMenu);

		subMenuGapsBases.add(_itemOptionGapsBaseColored);
		subMenuGapsBases.add(_dashBasesColor);

		_itemShowNCBP.setActionCommand("shownc");
		_itemShowNCBP.addActionListener(_controleurMenu);
		_itemShowNCBP.setAccelerator(KeyStroke.getKeyStroke("control W"));

		_itemShowOnlyPlanar.setActionCommand("shownp");
		_itemShowOnlyPlanar.addActionListener(_controleurMenu);
		_itemShowOnlyPlanar.setAccelerator(KeyStroke.getKeyStroke("control E"));

		JMenuItem numPeriodDisplay = new JMenuItem("Num. periodicity");
		numPeriodDisplay.setActionCommand("numPeriod");
		numPeriodDisplay.setAccelerator(KeyStroke.getKeyStroke("control M"));
		numPeriodDisplay.setMnemonic(KeyEvent.VK_M);
		numPeriodDisplay.addActionListener(_controleurMenu);

		JMenuItem backboneDisplay = new JMenuItem("Backbone Color");
		backboneDisplay.setActionCommand("backbone");
		backboneDisplay.setAccelerator(KeyStroke.getKeyStroke("control K"));
		backboneDisplay.setMnemonic(KeyEvent.VK_K);
		backboneDisplay.addActionListener(_controleurMenu);
		_disabled.add(backboneDisplay);

		JMenuItem backgroundDisplay = new JMenuItem("Background Color");
		backgroundDisplay.setActionCommand("background");
		backgroundDisplay.setAccelerator(KeyStroke.getKeyStroke("control G"));
		backgroundDisplay.setMnemonic(KeyEvent.VK_G);
		backgroundDisplay.addActionListener(_controleurMenu);

		JSeparator sep = new JSeparator();
		_disabled.add(sep);

		JMenu subMenuBasePairs = new JMenu("Base Pairs");

		subMenuBasePairs.add(submenuBPStyle);
		subMenuBasePairs.add(_itemShowNCBP);
		subMenuBasePairs.add(_itemShowOnlyPlanar);

		submenuStyle.add(subMenuBasePairs);
		submenuStyle.addSeparator();
		submenuStyle.add(subMenuNSBases);
		// submenuDisplay.add(_itemOptionSpecialBaseColored);
		// submenuDisplay.add(specialBasesColor);
		submenuStyle.add(subMenuGapsBases);
		submenuStyle.add(sep);
		submenuStyle.add(backboneDisplay);

		subMenuMisc.add(backgroundDisplay);
		subMenuMisc.add(numPeriodDisplay);
		subMenuMisc.add(subMenuTitle);

		add(submenuStyle);
		add(subMenuMisc);

	}

	private void redrawMenu() {
		JMenu submenuRedraw = new JMenu("Redraw");
		_disabled.add(submenuRedraw);

		JMenu submenuPlacement = new JMenu("Algorithm");
		_disabled.add(submenuPlacement);

		JMenuItem algoCircle = new JMenuItem("Circular");
		algoCircle.setActionCommand("circular");
		algoCircle.setAccelerator(KeyStroke.getKeyStroke("control 2"));
		algoCircle.addActionListener(_controleurMenu);
		_disabled.add(algoCircle);

		JMenuItem algoRadiate = new JMenuItem("Radiate");
		algoRadiate.setActionCommand("radiate");
		algoRadiate.setAccelerator(KeyStroke.getKeyStroke("control 3"));
		algoRadiate.addActionListener(_controleurMenu);
		_disabled.add(algoRadiate);

		JMenuItem algoNAView = new JMenuItem("NAView");
		algoNAView.setActionCommand("naview");
		algoNAView.setAccelerator(KeyStroke.getKeyStroke("control 4"));
		algoNAView.addActionListener(_controleurMenu);
		_disabled.add(algoNAView);

		JMenuItem algoLine = new JMenuItem("Linear");
		algoLine.setActionCommand("line");
		algoLine.setAccelerator(KeyStroke.getKeyStroke("control 1"));
		algoLine.addActionListener(_controleurMenu);
		_disabled.add(algoLine);

		submenuPlacement.add(algoLine);
		submenuPlacement.add(algoCircle);
		submenuPlacement.add(algoRadiate);
		submenuPlacement.add(algoNAView);

		// Item pour le rglage de l'espace entre chaques bases
		JMenuItem spaceBetweenBases = new JMenuItem("Space between bases");
		spaceBetweenBases.setActionCommand("spaceBetweenBases");
		spaceBetweenBases.setMnemonic(KeyEvent.VK_S);
		spaceBetweenBases.setAccelerator(KeyStroke
				.getKeyStroke("control shift S"));
		spaceBetweenBases.addActionListener(_controleurMenu);

		// Sets the height increment in LINE_MODE type of drawing
		_bpHeightIncrement = new JMenuItem("BP height increment");
		_bpHeightIncrement.setActionCommand("bpheightincrement");
		_bpHeightIncrement.setAccelerator(KeyStroke.getKeyStroke("control H"));
		_bpHeightIncrement.setMnemonic(KeyEvent.VK_H);
		_bpHeightIncrement.addActionListener(_controleurMenu);
		_bpHeightIncrement
				.setEnabled((_vp.getRNA().get_drawMode() == RNA.DRAW_MODE_LINEAR));

		_itemFlatExteriorLoop.setActionCommand("flat");
		_itemFlatExteriorLoop.setAccelerator(KeyStroke
				.getKeyStroke("control F"));
		_itemFlatExteriorLoop.addActionListener(_controleurMenu);
		_disabled.add(_itemFlatExteriorLoop);

		JMenuItem redisplay = new JMenuItem("Reset");
		redisplay.setActionCommand("reset");
		redisplay.setAccelerator(KeyStroke.getKeyStroke("control shift R"));
		redisplay.addActionListener(_controleurMenu);

		submenuRedraw.add(submenuPlacement);
		submenuRedraw.add(_bpHeightIncrement);
		submenuRedraw.add(_itemFlatExteriorLoop);
		submenuRedraw.add(spaceBetweenBases);
		submenuRedraw.add(redisplay);
		add(submenuRedraw);
	}

	@SuppressWarnings("unused")
	private void warningMenu() {
		// Menu showWarning
		if (!_vp.isModifiable())
			_itemShowWarnings.setEnabled(false);
		_itemShowWarnings.setActionCommand("showwarnings");
		_itemShowWarnings.addActionListener(_controleurMenu);
		add(_itemShowWarnings);
	}

	private void viewMenu() {
		// View menu
		JMenu subMenuView = new JMenu("View");

		JMenuItem borderDisplay = new JMenuItem("Border size");
		borderDisplay.setMnemonic(KeyEvent.VK_B);
		borderDisplay.setAccelerator(KeyStroke.getKeyStroke("control B"));
		borderDisplay.setActionCommand("borderSize");
		borderDisplay.addActionListener(_controleurMenu);

		// Item pour le choix de l'intensité du zoom
		JMenu zoomDisplay = new JMenu("Zoom");
		JMenuItem zoomQuarterDisplay = new JMenuItem("25%");
		zoomQuarterDisplay.setActionCommand("zoom25");
		zoomQuarterDisplay.addActionListener(_controleurMenu);
		zoomDisplay.add(zoomQuarterDisplay);
		JMenuItem zoomHalfDisplay = new JMenuItem("50%");
		zoomHalfDisplay.setActionCommand("zoom50");
		zoomHalfDisplay.addActionListener(_controleurMenu);
		zoomDisplay.add(zoomHalfDisplay);
		JMenuItem zoomWholeDisplay = new JMenuItem("100%");
		zoomWholeDisplay.setActionCommand("zoom100");
		zoomWholeDisplay.addActionListener(_controleurMenu);
		zoomDisplay.add(zoomWholeDisplay);
		JMenuItem zoomOneAndHalfDisplay = new JMenuItem("150%");
		zoomOneAndHalfDisplay.setActionCommand("zoom150");
		zoomOneAndHalfDisplay.addActionListener(_controleurMenu);
		zoomDisplay.add(zoomOneAndHalfDisplay);
		JMenuItem zoomTwiceDisplay = new JMenuItem("200%");
		zoomTwiceDisplay.setActionCommand("zoom200");
		zoomTwiceDisplay.addActionListener(_controleurMenu);
		zoomDisplay.add(zoomTwiceDisplay);
		JMenuItem customZoomDisplay = new JMenuItem("Customize...");
		customZoomDisplay.setAccelerator(KeyStroke.getKeyStroke("control Z"));
		customZoomDisplay.setMnemonic(KeyEvent.VK_Z);
		customZoomDisplay.setActionCommand("zoom");
		customZoomDisplay.addActionListener(_controleurMenu);
		zoomDisplay.add(customZoomDisplay);

		_rotation = new JMenuItem("Rotation ...");
		_rotation.setAccelerator(KeyStroke.getKeyStroke("control R"));
		_rotation.setMnemonic(KeyEvent.VK_R);
		_rotation.setActionCommand("rotation");
		_rotation.addActionListener(_controleurMenu);
		if (_vp.getRNA().get_drawMode() == RNA.DRAW_MODE_LINEAR)
			_rotation.setEnabled(false);

		subMenuView.add(zoomDisplay);
		subMenuView.add(_rotation);
		subMenuView.addSeparator();
		subMenuView.add(borderDisplay);
		add(subMenuView);

	}

	JMenu _subMenuBases;

	private Component _selectionMenuIndex = null;

	public void addSelectionMenu(JMenuItem s) {
		_selectionMenuIndex = s;
		_disabled.add(s);
		insert(s, getComponentCount() - 2);
	}

	public void removeSelectionMenu() {
		if (_selectionMenuIndex != null) {
			this.remove(_selectionMenuIndex);
			_selectionMenuIndex = null;
		}
	}

	private void colorClassesMenu() {
		// Menu Bases
		_subMenuBases = new JMenu("Colors");
		_disabled.add(_subMenuBases);

		JMenuItem kindBases = new JMenuItem("By Base");
		kindBases.setActionCommand("eachKind");
		kindBases.setAccelerator(KeyStroke.getKeyStroke("control U"));
		kindBases.addActionListener(_controleurMenu);
		_disabled.add(kindBases);
		_subMenuBases.add(kindBases);

		JMenuItem coupleBases = new JMenuItem("By BP");
		coupleBases.setActionCommand("eachCouple");
		coupleBases.addActionListener(_controleurMenu);
		coupleBases.setAccelerator(KeyStroke.getKeyStroke("shift U"));
		_disabled.add(coupleBases);
		_subMenuBases.add(coupleBases);

		JMenuItem allBases = new JMenuItem("By Position");
		allBases.setActionCommand("eachBase");
		allBases.setAccelerator(KeyStroke.getKeyStroke("alt U"));
		allBases.addActionListener(_controleurMenu);
		_disabled.add(allBases);
		_subMenuBases.add(allBases);

		add(_subMenuBases);
	}

	/**
	 * add default color options to a menu
	 */
	public void addColorOptions(JMenu submenu) {
		JMenuItem innerColor = new JMenuItem("Fill color");
		innerColor.setActionCommand(submenu.getActionCommand() + ",InnerColor");
		innerColor.addActionListener(_controleurMenu);

		JMenuItem nameColor = new JMenuItem("Label color");
		nameColor.setActionCommand(submenu.getActionCommand() + ",NameColor");
		nameColor.addActionListener(_controleurMenu);

		JMenuItem outlineColor = new JMenuItem("Stroke color");
		outlineColor.setActionCommand(submenu.getActionCommand()
				+ ",OutlineColor");
		outlineColor.addActionListener(_controleurMenu);

		JMenuItem bpColor = new JMenuItem("BP color");
		bpColor.setActionCommand(submenu.getActionCommand() + ",BPColor");
		bpColor.addActionListener(_controleurMenu);

		JMenuItem bpThickness = new JMenuItem("BP thickness");
		bpThickness.setActionCommand(submenu.getActionCommand()
				+ ",BPThickness");
		bpThickness.addActionListener(_controleurMenu);

		JMenuItem numberColor = new JMenuItem("Number color");
		numberColor.setActionCommand(submenu.getActionCommand()
				+ ",NumberColor");
		numberColor.addActionListener(_controleurMenu);

		submenu.add(innerColor);
		submenu.add(outlineColor);
		submenu.add(nameColor);
		submenu.addSeparator();
		submenu.add(bpColor);
		submenu.add(bpThickness);
		submenu.addSeparator();
		submenu.add(numberColor);
	}

	public static long getSerialVersionUID() {
		return serialVersionUID;
	}

	public VARNAPanel get_vp() {
		return _vp;
	}

	public ControleurMenu get_controleurMenu() {
		return _controleurMenu;
	}

	public JCheckBoxMenuItem get_itemOptionSpecialBaseColored() {
		return _itemOptionSpecialBaseColored;
	}

	public JCheckBoxMenuItem get_itemShowWarnings() {
		return _itemShowWarnings;
	}

	public JCheckBoxMenuItem get_itemOptionDashBaseColored() {
		return _itemOptionGapsBaseColored;
	}

	public void set_controleurMenu(ControleurMenu menu) {
		_controleurMenu = menu;
	}

	public void set_itemOptionSpecialBaseColored(
			JCheckBoxMenuItem optionSpecialBaseColored) {
		_itemOptionSpecialBaseColored = optionSpecialBaseColored;
	}

	public void set_itemShowWarnings(JCheckBoxMenuItem showWarnings) {
		_itemShowWarnings = showWarnings;
	}

	public void set_itemOptionDashBaseColored(
			JCheckBoxMenuItem optionDashBaseColored) {
		_itemOptionGapsBaseColored = optionDashBaseColored;
	}

	public JMenuItem get_rotation() {
		return _rotation;
	}

	public void set_rotation(JMenuItem _rotation) {
		this._rotation = _rotation;
	}

	public JCheckBoxMenuItem get_itemOptionBondsColored() {
		return _itemOptionBondsColored;
	}

	public void set_itemOptionBondsColored(JCheckBoxMenuItem optionBondsColored) {
		_itemOptionBondsColored = optionBondsColored;
	}

	public void addAnnotationMenu(JMenu menu) {

		JMenu menuAnnotation = new JMenu("Annotation");

		JMenuItem itemAdd = new JMenuItem("Add");
		itemAdd.setActionCommand(menu.getActionCommand() + "annotationadd");
		itemAdd.addActionListener(_controleurMenu);
		menuAnnotation.add(itemAdd);

		JMenuItem itemEdit = new JMenuItem("Edit");
		itemEdit.setActionCommand(menu.getActionCommand() + "annotationedit");
		itemEdit.addActionListener(_controleurMenu);
		menuAnnotation.add(itemEdit);

		JMenuItem itemRemove = new JMenuItem("Remove");
		itemRemove.setActionCommand(menu.getActionCommand()
				+ "annotationremove");
		itemRemove.addActionListener(_controleurMenu);
		menuAnnotation.add(itemRemove);

		menu.add(menuAnnotation);
	}

	public void addAnnotationMenuSansAdd(JMenu menu) {
		String debut = "";
		String texte = _vp.get_selectedAnnotation().getTexte();
		if (texte.length() < 5)
			debut = texte;
		else
			debut = texte.substring(0, 5) + "...";
		JMenu menuAnnotation = new JMenu("Annotation: " + debut);

		JMenuItem itemEdit = new JMenuItem("Edit");
		itemEdit.setActionCommand(menu.getActionCommand() + "annotationedit");
		itemEdit.addActionListener(_controleurMenu);
		menuAnnotation.add(itemEdit);

		JMenuItem itemRemove = new JMenuItem("Remove");
		itemRemove.setActionCommand(menu.getActionCommand()
				+ "annotationremove");
		itemRemove.addActionListener(_controleurMenu);
		menuAnnotation.add(itemRemove);

		menu.add(menuAnnotation);
	}

	public void show(Component invoker,int x,int y) {
		 _spawnOrigin = new Point(x,y);
		 super.show(invoker,x,y); 
	 }
	
	public Point getSpawnPoint()
	{
		return _spawnOrigin ;
	}

}